/* Program to print the first 10 lines of a file, assuming the
   file is available for reading by anybody.  This progam runs as
   root/Administrator. */

#include <sys/types.h>
#include <sys/stat.h>
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <err.h>
#include <pwd.h>
#include <unistd.h>

const char *unpriv_user = "nobody";
const unsigned int max_line = 80;
const unsigned int max_lines = 10;

void drop_privs(void)
{
    struct passwd *new;

    errno = 0;
    if ((new = getpwnam(unpriv_user)) == NULL) {
        if (errno == 0)
            errx(1, "drop_privs: User '%s' not in passwd file\n",
                 unpriv_user);
        else
            err(1, "drop_privs: getpwnam(%s) failed", unpriv_user);
    }
    endpwent();

    if (setgid(new->pw_gid) < 0)
        err(1, "drop_privs: set gid to %d failed", new->pw_gid);

    if (setuid(new->pw_uid) < 0)
        err(1, "drop_privs: set uid to %d failed", new->pw_uid);
}

int main(int argc, char *argv[])
{
    struct stat sb;
    FILE *f;
    char line[max_line];
    int nlines;

    if (argc != 2)
        errx(1, "Usage: %s file\n", argv[0]);

    if (stat(argv[1], &sb) < 0)
        err(1, "Cannot stat '%s'", argv[1]);

    if (! sb.st_mode & S_IROTH)
        errx(1, "No other read permission on '%s'", argv[1]);

    if ((f = fopen(argv[1], "r")) == NULL)
        err(1, "Open '%s' for reading failed", argv[1]);

    drop_privs();

    for (nlines=0; nlines < max_lines &&
                   fgets(line, max_line, f) != NULL; nlines++) {
	int len;

	/* kill \n at end if it is there */
	len = strlen(line) - 1;
	if (line[len] == '\n') line[len] = '\0';

        printf("%d: %s\n", nlines+1, line);
    }

    return 0;
}
