/*
   Build a simple PNG image.  Based on an example from 
   http://www.boutell.com/gd/manual2.0.33.html

   Fixed the integer overflow bug.
*/


#include <gd.h>
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <getopt.h>
#include <inttypes.h>
#include <limits.h>

int main(int argc, char *argv[]) {
    FILE *pngout;
    gdImagePtr im;
    int black, white;
    u_int32_t x, y;
    u_int64_t pixels;

    /* arg processing */
    x = 64; y = 64; /* default values */
    while (1) {
        int c;

        c = getopt(argc, argv, "x:y:");
        if (c == -1) break;
        switch (c) {
            case 'x':
                x = strtol(optarg, NULL, 10);
                break;
            case 'y':
                y = strtol(optarg, NULL, 10);
                break;
        }
    }

    pixels = (u_int64_t)x * (u_int64_t)y;
    if (pixels > UINT_MAX) {
        fprintf(stderr, "x (%ud) or y (%ud) or both are too large.\n", x, y);
        fprintf(stderr, "The product exceeds UINT_MAX.\n");
        exit(1);
    }
    printf("Creating a %ud x %ud image (%ld pixels)\n", x, y, pixels);

    /* create the image and the colors we will use. */
    im = gdImageCreate(x, y);
    black = gdImageColorAllocate(im, 0, 0, 0);  
    white = gdImageColorAllocate(im, 255, 255, 255);  

    /* Draw a line from the upper left to the lower right,
       using white color index. */
    gdImageLine(im, 0, 0, x-1, y-1, white);  

    /* save the result as a PNG file */
    pngout = fopen("test.png", "wb");
    gdImagePng(im, pngout);
    fclose(pngout);

    return 0;
}
