/* A program with one or more stack overflow bugs. */
/* fixed version */

#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <err.h>
#include <time.h>
#include <alloca.h>
#include <string.h>
#include <fcntl.h>
#include <stdlib.h>
#include "string.h" /* CHANGE: for strlcpy definition.  May not be needed */

const char *dbname = "userdb";

#define N 80
#define W 120
#define D 27

/* CHANGE: date uses a constant and is now the correct length. */
struct db_entry {
    char name[N];
    char date[D];
};

struct db_st {
    int fd;
    struct db_entry *dbp;
    unsigned nelem;
};

/* CHANGE: new function */
struct db_entry* make_db_entry(const char *name, const char *date)
{
    struct db_entry *new;

    if ((new = calloc(1, sizeof(struct db_entry))) == NULL)
        err(1, "calloc failed in make_db_entry");

    strlcpy(new->name, name, N);
    strlcpy(new->date, date, D);
    return new;
}

/* Read in the database */
void dbinit(struct db_st *db)
{
    struct stat sb;

    if ((db->fd = open(dbname, O_RDWR|O_CREAT, 0666)) == -1)
        err(1, "Unable to open %s", dbname);
    if (fstat(db->fd, &sb) < 0)
        err(1, "Unable to fstat opened file %s", dbname);

    /* CHANGE: added sanity check */
    if (sb.st_size % sizeof(struct db_entry) != 0)
        errx(1, "Database size is not an even multiple of %u\n",
             sizeof(struct db_entry));

    db->nelem = sb.st_size / sizeof(struct db_entry);
    /* CHANGE: simplify; we do not need size variable */
    
    if (sb.st_size > 0) {
        int i;

        /* CHANGE: use calloc, not alloca */
        db->dbp = (struct db_entry *)calloc(db->nelem, sizeof(struct db_entry));
        if (db->dbp == NULL)
            err(1, "calloc for db failed");

        if (read(db->fd, db->dbp, sb.st_size) < 0)
            err(1, "Read from %s failed", dbname);
    } else {
        db->nelem = 0;
        db->dbp = NULL;
    }
}

/* lookup a name */
char *lookup(struct db_st *db, char *name)
{
    struct db_entry *dbe;
    char *notfound = "Never";
    int i;
    
    /* CHANGE: use a loop index, not pointer indexing */
    dbe = db->dbp;
    for (i=0; i < db->nelem; i++) {
       if (strcmp(name, dbe[i].name) == 0)
           return dbe[i].date;
    }
    return notfound;
}

/* store the name and time */
void store(struct db_st *db, char *name)
{
    time_t now;
    char *now_str;
    struct db_entry *dbe, *newdb;

    now = time(NULL);
    now_str = ctime(&now);
    
    if (db->nelem == 0) {
        /* first element; create it. */
        db->dbp = make_db_entry(name, now_str);
        db->nelem = 1;
    } else {
        int i;

        /* db exists; search for it */
        /* CHANGE: index loop */
        for (dbe = db->dbp, i=0; i < db->nelem; i++) {
            if (strcmp(name, dbe[i].name) == 0) { 
                /* found it */
                strcpy(dbe[i].date, now_str);
                return;
            }
        }
        /* if we are here, the entry is new */
        /* CHANGE: use realloc */
        db->dbp = realloc(db->dbp, (db->nelem + 1) * sizeof(struct db_entry));
        if (db->dbp == NULL)
            err(1, "realloc failed");

        /* CHANGE: use strlcpy, put entry at end */
        strlcpy(db->dbp[db->nelem].name, name, N);
        strlcpy(db->dbp[db->nelem].date, now_str, D);
        (db->nelem)++;
    }
}

void dbclose(struct db_st *db)
{
    if (lseek(db->fd, 0, SEEK_SET) < 0)
        err(1, "lseek failed");

    if (write(db->fd, db->dbp, db->nelem * sizeof(struct db_entry)) < 0)
        err(1, "write to %s failed", dbname);

    close(db->fd);
}

int main(int argc, char *argv[])
{
    char name[N]; /* user's name */
    char welcome[W];
    struct db_st db;

    dbinit(&db);

    printf("Enter your name: "); fflush(stdout);
    /* CHANGE: use fgets, not gets */
    name[N-1] = '\0';
    if (fgets(name, N-1, stdin) == NULL)
        errx(1, "EOF while reading name");
    /* remove trailing \n */
    name[strlen(name)-1] = '\0';

    snprintf(welcome, W-1, "Welcome to my progam, %s", name);
    welcome[W-1] = '\0'; /* paranoia */

    puts(welcome);

    printf("You were last here: %s\n", lookup(&db, name));

    store(&db, name);
    dbclose(&db);

    return 0;
}
